;*****************************************
;*  Reiner Ziegler GB Flashprogram V1.0  *
;*****************************************
;
;  Compiled with RGBASM V1.10.
;
;  last edit 07-June-99 by Reiner Ziegler
;
;  1.0   first version
;
; Achtung Bank 0 ntig fr Serial Int Routine !!!!
;
;*****************************************

	INCLUDE "memory1.asm"
	INCLUDE "keypad.asm"
	INCLUDE "ibmpc1.inc"

;Version 1.1 or later of 'memory1.asm' is required.
	rev_Check_memory1_asm 1.1

;Version 1.2 or later of 'keypad.asm' is required.
	rev_Check_keypad_asm 1.2

;Version 1.1 or later of 'ibmpc1.inc' is required.
	rev_Check_ibmpc1_inc 1.1

GoRom           EQU     $C002
TXIDLE          EQU     $80             ; Serial idle tx character

	SECTION "Low Ram",BSS

UtilityVars     DS      2               ; used by keypad.asm
ExecStore       DS      800             ; Storage for ExecStart routine
GBtype          DS      1               ; GameBoy type DMG,MGB,CGB
GBcart          DS      1               ; selected cartbank
GBmbc           DS      1               ; MBC1 register
GBaddr          DS      2               ; Address
GBlength        DS      2               ; data length
GBdata          DS      2048            ; Storage for programming data
rxchar          DS      1               ; serial receive storage
txchar          DS      1               ; serial transmit storage
rxflag          DS      1               ; Flag for byte received
txflag          DS      1		; Flag for send byte
 	
	SECTION "Org $50",HOME[$50]     ; Timer Overflow Interrupt
        reti

	SECTION "Org $58",HOME[$58]     ; Serial Transfer Interrupt
	push    af

        ld      a,1                     ; flag for
        ld      [rxflag],a              ; key received

        ldh     a,[1]                   ; $FF01 transfer data
	ld      [rxchar],a              ; Save received char

	ld      a,[txflag]		; send byte ?
	cp      a,$01
	jp      nz,send_ok
	ld      a,$00
	ld      [txflag],a
	ld      a,[txchar]
	jp      send_new
send_ok:
	ld      a,[rxchar]
send_new:
	ldh     [1],a                   ; resend data
	ld      a,$80                   ; use external clock
        ldh     [2],a                   ; $FF02 serial control        

	pop     af
	reti

	SECTION "Org $D0",HOME[$D0]
	
	DB      "Reiner Ziegler   "
        DB      "Date ??.???.1999 "

	SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

	nop
	jp      begin

	NINTENDO_LOGO

;Rom Header Info

        db "GB-FLASH V1.0   "        ; Cart name   16bytes
        db 0,0,0                     ; Licensee codes
        db 3                         ; Cart type   ROM+MBC1+RAM+Battery
        db 0                         ; ROM Size    32K
        db 3                         ; RAM Size    32K
        db 1,1                       ; Maker ID
        db 0                         ; Version = 0
        db $56                       ; Complement check (important)
        dw $8d30                     ; Checksum (not important)

begin:
	di
			       ; The stack initializes to $FFFE
        ld     [GBtype],a      ; save Gameboy type
	xor    a               ; a = 0
	ldh    [rIF],a         ; clear pending interrupts

	ld     a,8
	ldh    [rIE],a         ; enable serial interrupt

	ld     a,0
	ldh    [$26],a         ; kill all sounds
	ld     [txflag],a
	ld     [rxflag],a

	ld     a,TXIDLE        ; Clear serial rxchar & txchar
	ld     [rxchar],a
	ld     [txchar],a

	ei                     ; turn on interrupts

	ld     a,[GBtype]      ; initiate continuous serial transfer
        ldh    [1],a
	ld     a,$80           ; use external clock
        ldh    [2],a

	lcd_WaitVRAM           ; Must be in VBL before turning the screen off.

	ld     a,%00010101     ; LCD Controller = Off [No picture on screen]
			       ; WindowBank = $9800 [Not used]
			       ; Window = OFF
			       ; BG Chr = $8000
			       ; BG Bank= $9800
			       ; OBJ    = 8x16
			       ; OBJ    = Off
			       ; BG     = On
	ldh     [rLCDC],a

; Load char set

	ld      bc,TileData
	call    Move_Char        ; Move the charset to $8000

	ld      a,$e4
	ldh     [rBGP],a

; Clear Screen

	ld      hl,$9800
	ld      bc,32*18
	ld      a,32
	call    mem_Set

	ld      hl,MakeSel
	ld      de,$9800+(32)
	ld      bc,MakeSelEnd-MakeSel
	call    mem_Copy

        ld      a,[GBtype]
        cp      1
        jp      nz,SG2
        ld      hl,GB_SG
        jp      LCD_on
SG2:    cp      $ff
        jp      nz,CG
        ld      hl,PG_SG2
        jp      LCD_on
CG:     cp      $11
        jp      nz,no_GB
        ld      hl,CGB
	jp      LCD_on
no_GB:	ld      hl,Wrong

LCD_on: ld      de,$9800+(5*32)
        ld      bc,32
        call    mem_Copy

	ld      a,%10010101     ; LCD Controller = On
	ldh     [rLCDC],a

; Load RAM bootstrap ROM exec routine

	ld      hl,ExecStart
	ld      de,GoRom
	ld      bc,ExecEnd-ExecStart
	call    mem_Copy
;        call    GoRom                  ; start serial transfer program

; This routine gets loaded into RAM and
; starts serial transfer program.

ExecStart:
        ld      hl,NoSync
        call    Draw_Mess

ns:     call    inchar
        cp      $40             ; @ received ?
        jr      nz,ns
ns2:    call    inchar          ; @ received ?
        cp      $40
        jr      nz,ns2

        ld      hl,Sync
        call    Draw_Mess

        call    inchar
        ld      [GBcart],a
        call    inchar
        ld      [GBmbc],a
        call    inchar
        ld      [GBaddr],a
        call    inchar
        ld      [GBaddr+1],a
        call    inchar
        ld      [GBlength],a
        call    inchar
        ld      [GBlength+1],a
        
ns3:    call    inchar
        cp      $40             ; @ received ?
        jr      nz,ns3

        ld      a,[GBcart]
        cp      $F0             ; erase chip ?
        jr      nz,noclr
        call    Erase_Chip
        ld      a,[GBtype]
        ldh     [1],a
        jr      ExecStart
noclr:  cp      $F1             ; next cartbank ?
        jr      nz,no_cb
        ld      a,[GBtype]
        ldh     [1],a
        jr      ExecStart
no_cb:  cp      $F2             ; reset module ?
        jr      nz,no_rst

        jp      begin

no_rst: cp      $F3		; erase sector ?
	jr      nz,no_sec
        ld      a,[GBmbc]	; sector number
	call    Erase_Sector	
        jr      ExecStart
no_sec: ld      hl,Read
        call    Draw_Mess

        ld      hl,GBdata
	ld      a,[GBlength]
        ld      b,a
        ld      a,[GBlength+1]
        ld      c,a
 
inloop: call    inchar
	
        ld      [hl+],a
no_in:  dec     c
	jr      nz,inloop
	dec     b
	jr      nz,inloop

ns4:    call    inchar          ; @ received ?
        cp      $40
        jr      nz,ns4

        ld      hl,Prog
        call    Draw_Mess
        call    Program_Chip

ns5:    call    inchar          ; @ received ?
        cp      $40
        jr      nz,ns5

;       jr      ExecStart
        ld      hl,GoRom
        jp      [hl]
        
;************************************
;*
;* read character via GB Link
;*
;************************************

inchar: ld      a,[rxflag]
        cp      1
        jr      nz,inchar

        xor     a
        ld      [rxflag],a
        ld      a,[rxchar]
        ret

;************************************
;*
;* wait for flashchip ready
;*
;************************************

wait_ready:
	ld      hl,$0	; read status
	ld      a,[hl]

	bit     7,a
	jr	z,wait_end	; ready ?
	ld      a,[hl]
	bit     5,a		
	jr	z,wait_ready	; timeout ?
	ld      a,[hl]
	bit     7,a
	jr	z,wait_end	; ready ?
	jr      wait_error

; old part of routine

	ld      hl,$0	; read status
	ld      a,[hl]

	push    af
	and     a,$80
        cp      a,$80   ; end of erase
        jr      z,wait_end

	pop	af
	and     a,$20
        cp      a,$20   ; exceeded time limit
        jr      z,wait_error
	
	jr      wait_ready
         
wait_error:
	ld      a,$01
	ret
wait_end: 
	ld      a,$00
        ret

;************************************
;*
;* erase total chip
;*
;************************************

Erase_Chip:
        push    af

        ld      hl,$555
        ld      a,$aa
        ld      [hl],a

        ld      hl,$2aa
        ld      a,$55
        ld      [hl],a

        ld      hl,$555
        ld      a,$80
        ld      [hl],a

        ld      hl,$555
        ld      a,$aa
        ld      [hl],a

        ld      hl,$2aa
        ld      a,$55
        ld      [hl],a

        ld      hl,$555
        ld      a,$10
        ld      [hl],a
        
        call    wait_ready

        pop     af
        ret

;************************************
;*
;* erase sector 
;* input a = sector number
;*
;************************************

Erase_Sector:
        push    af

        ld      hl,$555
        ld      a,$aa
        ld      [hl],a

        ld      hl,$2aa
        ld      a,$55
        ld      [hl],a

        ld      hl,$555
        ld      a,$80
        ld      [hl],a

        ld      hl,$555
        ld      a,$aa
        ld      [hl],a

        ld      hl,$2aa
        ld      a,$55
        ld      [hl],a

	pop     af
        ld      hl,$0   ; clear register
        ld      l,a     ; load sector number
        ld      a,$30
        ld      [hl],a
        
        call    wait_ready

        ret

;************************************
;*
;* Program flash chip with
;* 2048 bytes
;*
;************************************

Program_Chip:

        ld      hl,GBdata		; pointer to data			
	push    hl

	ld      a,[GBlength]		; length counter init
        ld      b,a
        ld      a,[GBlength+1]
        ld      c,a

	ld      hl,$6000		; select ROM16_RAM8
	ld      a,$0
	ld	[hl],a
	ld      hl,$4000		; store Cartbank 
        ld      a,[GBcart]         
	ld	[hl],a
	ld      hl,$6000		; select ROM4_RAM32, protect register
	ld      a,$1
	ld	[hl],a			 
	ld      hl,$2100		; store MBC bank 
        ld      a,[GBmbc]  
	ld	[hl],a

Program_Loop:

        ld      hl,$555
        ld      a,$aa
        ld      [hl],a

        ld      hl,$2aa
        ld      a,$55
        ld      [hl],a

        ld      hl,$555
        ld      a,$A0
        ld      [hl],a

	pop	hl			; get data
        ld      a,[hl+]
	ld      d,a			; save original data
        push    hl                      ; save data pointer

;       ld      hl,[GBaddr]             ; get address

        ld      hl,GBaddr
        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a

        ld      a,d                     ; restore data
        ld      [hl],a                  ; write data to address

	push    hl
	ld      hl,$6000		; select ROM16_RAM8
	ld      a,$0
	ld	[hl],a
	ld      hl,$4000		; store Cartbank 
        ld      a,[GBcart]         
	ld	[hl],a
	ld      hl,$6000		; select ROM4_RAM32, protect register
	ld      a,$1
	ld	[hl],a			 
	ld      hl,$2100		; store MBC bank 
        ld      a,[GBmbc]  
	ld	[hl],a
	pop	hl

Program_verify:

	ld      a,[hl]			; read data
	cp      a,d	
	jr      nz,Program_check
        jr      Program_next

Program_check:

	and     a,$20
        cp      a,$20   		; exceeded time limit
        jr      z,Program_Error
	jr      Program_verify

Program_next:

	inc     hl
;        ld      [GBaddr],hl             ; save address pointer



	dec     c
	jr      nz,Program_Loop
	dec     b
	jr      nz,Program_Loop

	ld      a,$00
        ret

Program_Error:
	ld      a,$01
	ret

;************************************
;*
;* messages
;*
;************************************

Sync:  	DB      " sync found !                   "
NoSync: DB      " no sync ...                    "
Read:   DB      " reading ...                    "
Prog:   DB      " programming ...                "    
Error:  DB      " prgr. error                    "

GB_SG:  DB      " for GameBoy or SGB             "  ; 01h
PG_SG2: DB      " for PocketGB or SG2            "  ; ffh
CGB:    DB      " for ColorGB                    "  ; 11h
Wrong:  DB      " for unknown device             "

;***************************************************************************
;*
;* draw messages on GB screen
;*
;* input  hl - source  
;*
;***************************************************************************

Draw_Mess:
	lcd_WaitVRAM            ; Must be in VBL before turning the screen off
	ld      a,%00010101    
	ldh     [rLCDC],a

        ld      de,$9800+(8*32)
	ld      bc,32
	
	inc     b               ; mem_copy
	inc     c
	jr      .skip
.loop   ld      a,[hl+]
	ld      [de],a
	inc     de
.skip   dec     c
	jr      nz,.loop
	dec     b
	jr      nz,.loop

	ld      a,%10010101    
	ldh     [rLCDC],a
	ret
ExecEnd:

; move charset to $8000

Move_Char:
	ld     hl,$8000
	ld     de,$1000
.lp1
	ld     a,[bc]
	ld     [hl+],a
	ld     [hl+],a
	inc    bc
	dec    e
	jr     nz,.lp1
	dec    d
	jr     nz,.lp1
	ret

TileData:
	chr_IBMPC1 1,4          ; include IBMPC char set

MakeSel:
	DB      " GB-FLASH Programmer            "
        DB      " V1.0 (c) 2000 by               "
        DB      " Reiner Ziegler                 "
MakeSelEnd:
        DS      $3000
